! Time-stamp: <fe1.fsl.noaa.gov:/home/jacobson/TM5/proj/ctemis/branches/oif/src/emission_co2_ocean__OIF.F90: 21 Sep 2011 (Wed) 01:04:49 GMT>

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Compute air-sea exchange of CO2 in mol m-2 s-1.  Positive is source to atmosphere.
!       This version uses the Jacobson et al. ocean inversion fluxes (OIF)

module emission_co2_ocean
  
  use GO,		only: gol, goErr, goPr, goLabel
  use Dims,		only: nregions, nlon360,nlat180
  use Meteo,		only: Set
  use emission_common,	only: enkf_parms, co2_ocn, EmisInputDir, field_1x1
  use emission_common,	only: nmembersloc

  implicit none

  character(len=*), parameter	:: mname = 'emission_co2_ocn'

  private

  public	:: emission_co2_ocean_init, emission_co2_ocean_done, emission_co2_ocean_calc

  ! use this to potentially override the EmisInputDir
  character(len=2048)		:: OceanInputDir

  ! file name prefix
  character(len=1024)		:: deltapco2_prefix

contains

  subroutine emission_co2_ocean_init(status)

    use GO,		only : TrcFile, Init, Done, ReadRc
    use global_data,	only : rcfile
    use dims,		only : im,jm,nregions, iglbsfc
    use Meteo,		only : u10m_dat, v10m_dat, ci_dat


    implicit none

    character(len=*), parameter ::  rname = mname//'/emission_co2_ocean_init'

    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'ocean.input.dir', OceanInputDir ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'deltapco2.prefix', deltapco2_prefix ,status)
    IF_NOTOK_RETURN(status=1)
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    call Set(   u10m_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)
    call Set(   v10m_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)
    call Set(   ci_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_ocean_init]	ocean input files to be read from """,a,""", with delta_pco2 prefix of """,a,""".")') trim(OceanInputDir), trim(deltapco2_prefix)
    call goPr

  end subroutine emission_co2_ocean_init

  subroutine emission_co2_ocean_calc(parms,co2_ocn,flux_updated,status)

    use Dims,			only : nlon360, nlat180, idate, itau, nread, newmonth, iglbsfc
    use Dims,			only : sec_month,sec_year
    use emission_data,		only : flux_to_gridbox_per_month
    use MDF,			only : MDF_HDF4
    use ReadFromFile_MDF,	only : ReadFromFile
    use chem_param,		only : xmc
    use Meteo,			only : u10m_dat, v10m_dat, ci_dat
    use go_string,		only: goNum2str


    implicit none
    
    character(len=*), parameter			:: rname = mname//'/emission_co2_ocean_calc'

    type(field_1x1), dimension(:), intent(out)	:: co2_ocn
    type(enkf_parms), dimension(:), intent(in)	:: parms
    logical, intent(out)			:: flux_updated

    integer, intent(out)			:: status
    integer					:: i,j,ii,jj,imember
    real					:: month2year
    real,dimension(nlon360,nlat180)		:: ww,k92,schmidtno,solubility,deltapco2,flux
    real,pointer,dimension(:,:)			:: u10m,v10m,ci

    flux_updated = .false.

    if(mod(itau,nread) /= 0) return  ! only every nread hours

    flux_updated = .true.

    ! get necessary fields

    if(newmonth) then

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(deltapco2_prefix)//'.'//trim(goNum2str(idate(1)))//'.'//trim(goNum2str(idate(2),'(i2.2)'))//'.hdf', &
            ftype=MDF_HDF4,vname='DELTAPCO2', buffer=deltapco2, status=status)
       IF_NOTOK_RETURN(status=1)

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(deltapco2_prefix)//'.'//trim(goNum2str(idate(1)))//'.'//trim(goNum2str(idate(2),'(i2.2)'))//'.hdf', &
            ftype=MDF_HDF4,vname='SOLUBILITY', buffer=solubility, status=status)
       IF_NOTOK_RETURN(status=1)

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(deltapco2_prefix)//'.'//trim(goNum2str(idate(1)))//'.'//trim(goNum2str(idate(2),'(i2.2)'))//'.hdf', &
            ftype=MDF_HDF4,vname='SCHMIDTNO', buffer=schmidtno, status=status)
       IF_NOTOK_RETURN(status=1)


    endif

    u10m => u10m_dat(iglbsfc)%data1(:,:,1)
    v10m => v10m_dat(iglbsfc)%data1(:,:,1)
    ci => ci_dat(iglbsfc)%data1(:,:,1)

    ! make diffusion in m/s according to Wanninkhof92 param on 1x1 degree

    k92=0.0
    ww=u10m**2+v10m**2  ! note: sqrt has been taken away since we square it in the next formula!
    where(schmidtno(:,:).ge.0) k92=(0.31*(ww)/sqrt(schmidtno/660.0))*1.e-2/3600.   ! only over oceans

    ! make fluxes of co2 in mol/m2/s with tm5 meteo.  1e-6 to change deltapco2 from uatm to atm.
    flux=0
    where((solubility(:,:).ge.0).and.(deltapco2(:,:).gt.-1e10)) flux=k92*solubility*deltapco2*1e-6
    flux=flux*(1.0-ci) ! multiply out the sea-ice for this day

    month2year = sec_year/sec_month

    do imember = 1,nmembersloc
       co2_ocn(imember)%surf = flux * parms(imember)%surf    ! apply parameters by member (forward has nmembersloc=1)

       ! co2_ocn now in mol m-2 s-1

       co2_ocn(imember)%surf=co2_ocn(imember)%surf*xmc/1.0e3 ! from mol m-2 s-1 to kgC m-2 s-1

       call flux_to_gridbox_per_month(co2_ocn(imember)%surf) ! convert from kgC m-2 s-1 to kgC 1x1_gridbox-1 month-1

       write (gol,'("[emission_co2_ocn_calc]	member ",i4,": oceanic emission global integral ",f12.4," PgC/yr.")') imember,sum(co2_ocn(imember)%surf)*1.0e-12*month2year
       call goPr

    end do

    nullify(u10m)
    nullify(v10m)
    nullify(ci)

    status=0

  end subroutine emission_co2_ocean_calc

  subroutine emission_co2_ocean_done(status)

    use dims,		only: nregions

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_ocean_done'

    status=0
    return

  end subroutine emission_co2_ocean_done

end module emission_co2_ocean
